/*
Copyright © 2025 Philipp Wolfer <phw@uploadedlobster.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package listenbrainz

import (
	"context"
	"time"

	"go.uploadedlobster.com/mbtypes"
	"go.uploadedlobster.com/musicbrainzws2"
	"go.uploadedlobster.com/scotty/internal/listenbrainz"
	"go.uploadedlobster.com/scotty/internal/models"
)

func LookupRecording(
	ctx context.Context,
	mb *musicbrainzws2.Client,
	mbid mbtypes.MBID,
) (*listenbrainz.Track, error) {
	filter := musicbrainzws2.IncludesFilter{
		Includes: []string{"artist-credits"},
	}
	recording, err := mb.LookupRecording(ctx, mbid, filter)
	if err != nil {
		return nil, err
	}

	artistMBIDs := make([]mbtypes.MBID, 0, len(recording.ArtistCredit))
	for _, artist := range recording.ArtistCredit {
		artistMBIDs = append(artistMBIDs, artist.Artist.ID)
	}
	track := listenbrainz.Track{
		TrackName:  recording.Title,
		ArtistName: recording.ArtistCredit.String(),
		MBIDMapping: &listenbrainz.MBIDMapping{
			// In case of redirects this MBID differs from the looked up MBID
			RecordingMBID: recording.ID,
			ArtistMBIDs:   artistMBIDs,
		},
	}
	return &track, nil
}

func AsListen(lbListen listenbrainz.Listen) models.Listen {
	listen := models.Listen{
		ListenedAt: time.Unix(lbListen.ListenedAt, 0),
		UserName:   lbListen.UserName,
		Track:      AsTrack(lbListen.TrackMetadata),
	}
	return listen
}

func AsLove(f listenbrainz.Feedback) models.Love {
	recordingMBID := f.RecordingMBID
	track := f.TrackMetadata
	if track == nil {
		track = &listenbrainz.Track{}
	}
	love := models.Love{
		UserName:      f.UserName,
		RecordingMBID: recordingMBID,
		Created:       time.Unix(f.Created, 0),
		Track:         AsTrack(*track),
	}

	if love.Track.RecordingMBID == "" {
		love.Track.RecordingMBID = love.RecordingMBID
	}

	return love
}

func AsTrack(t listenbrainz.Track) models.Track {
	track := models.Track{
		TrackName:        t.TrackName,
		ReleaseName:      t.ReleaseName,
		ArtistNames:      []string{t.ArtistName},
		Duration:         t.Duration(),
		TrackNumber:      t.TrackNumber(),
		DiscNumber:       t.DiscNumber(),
		RecordingMBID:    t.RecordingMBID(),
		ReleaseMBID:      t.ReleaseMBID(),
		ReleaseGroupMBID: t.ReleaseGroupMBID(),
		ISRC:             t.ISRC(),
		AdditionalInfo:   t.AdditionalInfo,
	}

	if t.MBIDMapping != nil && len(track.ArtistMBIDs) == 0 {
		for _, artistMBID := range t.MBIDMapping.ArtistMBIDs {
			track.ArtistMBIDs = append(track.ArtistMBIDs, artistMBID)
		}
	}

	return track
}
